package be.iminds.ilabt.jfed.util;

import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnection;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.conn.ssl.X509HostnameVerifier;
import org.apache.http.params.HttpParams;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.Socket;
import java.net.SocketImplFactory;
import java.security.*;

/**
 * SshProxyHelper:
 * uses some deprecated classes -> SSH proxy support is not an important feature, so not important if it breaks.
 */
public class SshServerProxyHelper {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(SshServerProxyHelper.class);

    private static class SshProxySocket extends Socket {
        public SshProxySocket(SocketImplFactory sshHSocketImplFactory) throws IOException {
            super(sshHSocketImplFactory.createSocketImpl());
            LOG.debug("Creating wrapping SshProxySocket");
        }
    }

//    public static class SshProxyInfo {
//        private final InetSocketAddress sshProxyAddr;
//        private final String sshUsername;
//        private final String sshKnownHostsFileLine;
//
//        private String pemPrivateKeyString;
//
//        /**
//         * @param sshKnownHostsFileLine The host key, as the single line it would appear in, in  a known_hosts file. The hostname is ignored but must be present (so it may be a dummy). The format of this line is: <hostname or hash of hostname> <key type (typically "ssh-rsa")> <host key in base64 encoding>
//         */
//        public SshProxyInfo(InetSocketAddress sshProxyAddr, String sshUsername, String sshKnownHostsFileLine, String pemPrivateKeyString) {
//            this.sshProxyAddr = sshProxyAddr;
//            this.sshUsername = sshUsername;
//            this.pemPrivateKeyString = pemPrivateKeyString;
//
//            this.sshKnownHostsFileLine = sshKnownHostsFileLine;
//        }
//
//        public InetSocketAddress getSshProxyAddr() {
//            return sshProxyAddr;
//        }
//
//        public String getHostName() {
//            return sshProxyAddr.getHostName();
//        }
//
//        public int getPort() {
//            return sshProxyAddr.getPort();
//        }
//
//        public String getSshUsername() {
//            return sshUsername;
//        }
//
//        public String getPemPrivateKeyString() {
//            return pemPrivateKeyString;
//        }
//
//        public char[] getPemPrivateKeyChars() {
//            return getPemPrivateKeyString().toCharArray();
//        }
//
//        public String getSshKnownHostsFileLine() {
//            return sshKnownHostsFileLine;
//        }
//
//        @Override
//        public String toString() {
//            return "ProxyHelper.SshProxyInfo{" +
//                    "sshProxyAddr=" + sshProxyAddr +
//                    ", sshUsername='" + sshUsername + '\'' +
//                    ", sshKnownHostsFileLine='" + sshKnownHostsFileLine + '\'' +
//                    '}';
//        }
//    }
//
//    public static SshProxyInfo transformSshProxyInfo(JFedConnection.SshProxyInfo sshProxyInfo){
//
//        return new SshServerProxyHelper.SshProxyInfo(
//                new InetSocketAddress(sshProxyInfo.getHostname(), sshProxyInfo.getPort()),
//                sshProxyInfo.getUsername(),
//                sshProxyInfo.getHostKey(),
//                new String(KeyUtil.privateKeyToAnyPem(sshProxyInfo.getSshKeyInfo().getPrivateKey()))
//        );
//    }


    public static class SslOverSshProxySocketFactory extends SSLSocketFactory {
        private JFedConnection.SshProxyInfo sshProxyInfo;

        protected final String forcedCreateSocketHostname;
        protected final Integer forcedCreateSocketPort;

        public SslOverSshProxySocketFactory(JFedConnection.SshProxyInfo sshProxyInfo,
                                            String algorithm,
                                            KeyStore keystore,
                                            String keystorePassword,
                                            KeyStore truststore,
                                            SecureRandom random,
                                            TrustStrategy trustStrategy,
                                            X509HostnameVerifier hostnameVerifier,
                                            String forcedCreateSocketHostname,
                                            Integer forcedCreateSocketPort)
                throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException, UnrecoverableKeyException {
            super(algorithm, keystore, keystorePassword, truststore, random, trustStrategy, hostnameVerifier);
            this.sshProxyInfo = sshProxyInfo;
            this.forcedCreateSocketHostname = forcedCreateSocketHostname;
            this.forcedCreateSocketPort = forcedCreateSocketPort;
        }

        @Override
        public Socket createSocket(final HttpParams params) {
            Socket socket;
            try {
                assert sshProxyInfo != null;

                SshProxySocketImplFactory sshHSocketImplFactory =
                        SshProxySocketImplFactory.getFactory(sshProxyInfo, forcedCreateSocketHostname, forcedCreateSocketPort);
                assert sshHSocketImplFactory != null;

                socket = new SshProxySocket(sshHSocketImplFactory);
                assert socket != null;
            } catch (IOException e) {
                throw new RuntimeException("Error while creating SshProxySocket", e);
            }

            return socket;
        }
    }

    public static class PlainHttpOverSshProxySocketFactory extends PlainSocketFactory {
        private JFedConnection.SshProxyInfo sshProxyInfo;
        protected final String forcedCreateSocketHostname;
        protected final Integer forcedCreateSocketPort;

        public PlainHttpOverSshProxySocketFactory(JFedConnection.SshProxyInfo sshProxyInfo) {
            super();
            this.sshProxyInfo = sshProxyInfo;
            this.forcedCreateSocketHostname = null;
            this.forcedCreateSocketPort = null;
        }
        public PlainHttpOverSshProxySocketFactory(JFedConnection.SshProxyInfo sshProxyInfo, String forcedCreateSocketHostname, Integer forcedCreateSocketPort) {
            super();
            this.sshProxyInfo = sshProxyInfo;
            this.forcedCreateSocketHostname = forcedCreateSocketHostname;
            this.forcedCreateSocketPort = forcedCreateSocketPort;
        }

        @Override
        public Socket createSocket(final HttpParams params) {
            Socket socket = null;
            try {
                SshProxySocketImplFactory sshHSocketImplFactory =
                        SshProxySocketImplFactory.getFactory(sshProxyInfo, forcedCreateSocketHostname, forcedCreateSocketPort);

                socket = new SshProxySocket(sshHSocketImplFactory);
            } catch (IOException e) {
                throw new RuntimeException("Error while creating SshProxySocket", e);
            }

            return socket;
        }
    }

}
